@@ -29,70 +29,72 @@
 #
 """
 
-    enchant.tokenize:    String tokenization functions for PyEnchant
+enchant.tokenize:    String tokenization functions for PyEnchant
+================================================================
 
-    An important task in spellchecking is breaking up large bodies of
-    text into their constituent words, each of which is then checked
-    for correctness.  This package provides Python functions to split
-    strings into words according to the rules of a particular language.
+An important task in spellchecking is breaking up large bodies of
+text into their constituent words, each of which is then checked
+for correctness.  This package provides Python functions to split
+strings into words according to the rules of a particular language.
     
-    Each tokenization function accepts a string as its only positional
-    argument, and returns an iterator that yields tuples of the following
-    form, one for each word found:
+Each tokenization function accepts a string as its only positional
+argument, and returns an iterator that yields tuples of the following
+form, one for each word found::
         
-        (<word>,<pos>)
+    (<word>,<pos>)
         
-    The meanings of these fields should be clear: <word> is the word
-    that was found and <pos> is the position within the text at which
-    the word began (zero indexed, of course).  The function will work
-    on any string-like object that supports array-slicing; in particular
-    character-array objects from the 'array' module may be used.
+The meanings of these fields should be clear: <word> is the word
+that was found and <pos> is the position within the text at which
+the word began (zero indexed, of course).  The function will work
+on any string-like object that supports array-slicing; in particular
+character-array objects from the 'array' module may be used.
     
-    The iterator also provides the attribute 'offset' which may be used
-    to get/set the current position of the tokenizer inside the string
-    being split.  This can be used for example if the string's contents
-    have changed during the tokenization process.
+The iterator also provides the attribute 'offset' which gives the current
+position of the tokenizer inside the string being split, and the method
+'set_offset' for manually adjusting this position.  This can be used for
+example if the string's contents have changed during the tokenization
+process.
     
-    To obtain an appropriate tokenization function for the language
-    identified by <tag>, use the function 'get_tokenizer(tag)':
+To obtain an appropriate tokenization function for the language
+identified by <tag>, use the function 'get_tokenizer(tag)'::
         
-        tknzr = get_tokenizer("en_US")
-        for (word,pos) in tknzr("text to be tokenized goes here")
-            do_something(word)
+    tknzr = get_tokenizer("en_US")
+    for (word,pos) in tknzr("text to be tokenized goes here")
+        do_something(word)
     
-    This library is designed to be easily extendible by third-party
-    authors.  To register a tokenization function for the language
-    <tag>, implement it as the function 'tokenize' within the
-    module enchant.tokenize.<tag>.  The 'get_tokenizer' function
-    will automatically detect it.  Note that the underscore must be
-    used as the tag component separator in this case, in order to
-    form a valid python module name. (e.g. "en_US" rather than "en-US")
+This library is designed to be easily extendible by third-party
+authors.  To register a tokenization function for the language
+<tag>, implement it as the function 'tokenize' within the
+module enchant.tokenize.<tag>.  The 'get_tokenizer' function
+will automatically detect it.  Note that the underscore must be
+used as the tag component separator in this case, in order to
+form a valid python module name. (e.g. "en_US" rather than "en-US")
     
-    Currently, a tokenizer has only been implemented for the English
-    language.  Based on the author's limited experience, this should
-    be at least partially suitable for other languages.
+Currently, a tokenizer has only been implemented for the English
+language.  Based on the author's limited experience, this should
+be at least partially suitable for other languages.
 
-    This module also provides various implementations of "Chunkers" and
-    "Filters".  These classes are designed to make it easy to work with
-    text in a vareity of common formats, by detecting and excluding parts
-    of the text that don't need to be checked.
+This module also provides various implementations of "Chunkers" and
+"Filters".  These classes are designed to make it easy to work with
+text in a variety of common formats, by detecting and excluding parts
+of the text that don't need to be checked.
 
-    A Chunker is a class designed to break a body of text into large chunks
-    of checkable content; for example the HTMLChunker class extracts the 
-    text content from all HTML tags but excludes the tags themselves.
-    A Filter is a class designed to skip individual words during the checking
-    process; for example the URLFilter class skips over any words that
-    have the format of a URL.
+A Chunker is a class designed to break a body of text into large chunks
+of checkable content; for example the HTMLChunker class extracts the 
+text content from all HTML tags but excludes the tags themselves.
+A Filter is a class designed to skip individual words during the checking
+process; for example the URLFilter class skips over any words that
+have the format of a URL.
 
-    For exmaple, to spellcheck an HTML document it is necessary to split the
-    text into chunks based on HTML tags, and to filter out common word forms
-    such as URLs and WikiWords.  This would look something like the following:
+For example, to spellcheck an HTML document it is necessary to split the
+text into chunks based on HTML tags, and to filter out common word forms
+such as URLs and WikiWords.  This would look something like the following::
 
-        tknzr = get_tokenier("en_US",(HTMLChunker,),(URLFilter,WikiWordFilter)))
+    tknzr = get_tokenizer("en_US",(HTMLChunker,),(URLFilter,WikiWordFilter)))
 
-        text = "<html><body>the url is http://example.com</body></html>"
-        for (word,pos) in tknzer(text):
-            ...check each word and react accordingly...
+    text = "<html><body>the url is http://example.com</body></html>"
+    for (word,pos) in tknzer(text):
+        ...check each word and react accordingly...
         
 """
 _DOC_ERRORS = ["pos","pos","tknzr","URLFilter","WikiWordFilter",
@@ -100,12 +102,13 @@ _DOC_ERRORS = ["pos","pos","tknzr","URLFilter","WikiWordFilter",
 
 import re
 import warnings
+import array
 
 import enchant
 from enchant.utils import next, xrange
 from enchant.errors import *
 
-#  For backwards-compatability.  This will eventually be removed, but how
+#  For backwards-compatibility.  This will eventually be removed, but how
 #  does one mark a module-level constant as deprecated?
 Error = TokenizerNotFoundError
 
@@ -124,7 +127,7 @@ class tokenize:
 
     def __init__(self,text):
         self._text = text
-        self.offset = 0
+        self._offset = 0
 
     def __next__(self):
         return self.next()
@@ -135,6 +138,17 @@ class tokenize:
     def __iter__(self):
         return self
 
+    def set_offset(self,offset,replaced=False):
+        self._offset = offset
+
+    def _get_offset(self):
+        return self._offset
+    def _set_offset(self,offset):
+        msg = "changing a tokenizers 'offset' attribute is deprecated;"\
+              " use the 'set_offset' method"
+        warnings.warn(msg,category=DeprecationWarning,stacklevel=2)
+        self.set_offset(offset)
+    offset = property(_get_offset,_set_offset)
 
 def get_tokenizer(tag=None,chunkers=None,filters=None):
     """Locate an appropriate tokenizer by language tag.
@@ -152,7 +166,7 @@ def get_tokenizer(tag=None,chunkers=None,filters=None):
     If a suitable function cannot be found, raises TokenizerNotFoundError.
     
     If given and not None, 'chunkers' and 'filters' must be lists of chunker
-    classes and filter classes resectively.  These will be applied to the
+    classes and filter classes respectively.  These will be applied to the
     tokenizer during creation.
     """
     if tag is None:
@@ -170,7 +184,7 @@ def get_tokenizer(tag=None,chunkers=None,filters=None):
                 if chunkers_are_filters:
                     msg = "passing 'filters' as a non-keyword argument "\
                           "to get_tokenizer() is deprecated"
-                    warnings.warn(msg,category=DeprecationWarning)
+                    warnings.warn(msg,category=DeprecationWarning,stacklevel=2)
                     filters = chunkers
                     chunkers = None
     # Ensure only '_' used as separator
@@ -244,7 +258,7 @@ class basic_tokenize(tokenize):
     
     def next(self):
         text = self._text
-        offset = self.offset
+        offset = self._offset
         while True:
             if offset >= len(text):
                 break
@@ -256,13 +270,13 @@ class basic_tokenize(tokenize):
             while offset < len(text) and not text[offset].isspace():
                 offset += 1
             ePos = offset
-            self.offset = offset
+            self._offset = offset
             # Strip chars from font/end of word
             while sPos < len(text) and text[sPos] in self.strip_from_start:
                 sPos += 1
             while 0 < ePos and text[ePos-1] in self.strip_from_end:
                     ePos -= 1
-            # Return if word isnt empty
+            # Return if word isn't empty
             if(sPos < ePos):
                 return (text[sPos:ePos],sPos)
         raise StopIteration()
@@ -309,7 +323,7 @@ class Chunker(tokenize):
     pass
  
 
-class Filter:
+class Filter(object):
     """Base class for token filtering functions.
     
     A filter is designed to wrap a tokenizer (or another filter) and do
@@ -338,8 +352,7 @@ class Filter:
         
         If this method returns true, the given word will be skipped by
         the filter.  This should be overridden in subclasses to produce the
-        desired functionality.  The default behaviour is not to skip any
-        words.
+        desired functionality.  The default behaviour is not to skip any words.
         """
         return False
 
@@ -382,55 +395,76 @@ class Filter:
         def next(self):
             # Try to get the next sub-token from word currently being split.
             # If unavailable, move on to the next word and try again.
-            try:
-                (word,pos) = next(self._curtok)
-                return (word,pos + self._curpos)
-            except StopIteration:
-                (word,pos) = next(self._tokenizer)
-                while self._skip(word):
+            while True:
+                try:
+                    (word,pos) = next(self._curtok)
+                    return (word,pos + self._curpos)
+                except StopIteration:
                     (word,pos) = next(self._tokenizer)
-                self._curword = word
-                self._curpos = pos
-                self._curtok = self._split(word)
-                return self.next()
-            
-        
+                    while self._skip(self._to_string(word)):
+                        (word,pos) = next(self._tokenizer)
+                    self._curword = word
+                    self._curpos = pos
+                    self._curtok = self._split(word)
+
+        def _to_string(self, word):
+            if type(word) is array.array:
+                if word.typecode == 'u':
+                    return word.tounicode()
+                elif word.typecode == 'c':
+                    return word.tostring()
+            return word
+
         # Pass on access to 'offset' to the underlying tokenizer.
-        def _getOffset(self):
+        def _get_offset(self):
             return self._tokenizer.offset
-        def _setOffset(self,val):
-            self._tokenizer.offset = val
-            # If we stay within the current word, also set on _curtok.
+        def _set_offset(self,offset):
+            msg = "changing a tokenizers 'offset' attribute is deprecated;"\
+                  " use the 'set_offset' method"
+            warnings.warn(msg,category=DeprecationWarning,stacklevel=2)
+            self.set_offset(offset)
+        offset = property(_get_offset,_set_offset)
+
+        def set_offset(self,val,replaced=False):
+            old_offset = self._tokenizer.offset
+            self._tokenizer.set_offset(val,replaced=replaced)
+            # If we move forward within the current word, also set on _curtok.
             # Otherwise, throw away _curtok and set to empty iterator.
-            subval = val - self._curpos
-            if subval >= 0 and subval < len(self._curword):
-                self._curtok.offset = subval
+            keep_curtok = True
+            curtok_offset = val - self._curpos
+            if old_offset > val:
+                keep_curtok = False
+            if curtok_offset < 0:
+                keep_curtok = False
+            if curtok_offset >= len(self._curword):
+                keep_curtok = False
+            if keep_curtok and not replaced:
+                self._curtok.set_offset(curtok_offset)
             else:
                 self._curtok = empty_tokenize()
                 self._curword = ""
                 self._curpos = 0
-        offset = property(_getOffset,_setOffset)
 
 
 #  Pre-defined chunkers and filters start here
 
 class URLFilter(Filter):
-    """Filter skipping over URLs.
+    r"""Filter skipping over URLs.
     This filter skips any words matching the following regular expression:
        
-           ^[a-zA-z]+:\/\/[^\s].*
+           ^[a-zA-Z]+:\/\/[^\s].*
         
     That is, any words that are URLs.
     """
     _DOC_ERRORS = ["zA"]
-    _pattern = re.compile(r"^[a-zA-z]+:\/\/[^\s].*")
+    _pattern = re.compile(r"^[a-zA-Z]+:\/\/[^\s].*")
     def _skip(self,word):
         if self._pattern.match(word):
             return True
         return False
 
 class WikiWordFilter(Filter):
-    """Filter skipping over WikiWords.
+    r"""Filter skipping over WikiWords.
     This filter skips any words matching the following regular expression:
        
            ^([A-Z]\w+[A-Z]+\w+)
@@ -444,7 +478,7 @@ class WikiWordFilter(Filter):
         return False
 
 class EmailFilter(Filter):
-    """Filter skipping over email addresses.
+    r"""Filter skipping over email addresses.
     This filter skips any words matching the following regular expression:
        
            ^.+@[^\.].*\.[a-z]{2,}$
@@ -457,6 +491,36 @@ class EmailFilter(Filter):
             return True
         return False
 
+class MentionFilter(Filter):
+    r"""Filter skipping over @mention.
+    This filter skips any words matching the following regular expression:
+       
+           (\A|\s)@(\w+)
+        
+    That is, any words that are @mention.
+    """
+    _DOC_ERRORS = ["zA"]
+    _pattern = re.compile(r"(\A|\s)@(\w+)")
+    def _skip(self,word):
+        if self._pattern.match(word):
+            return True
+        return False
+
+class HashtagFilter(Filter):
+    r"""Filter skipping over #hashtag.
+    This filter skips any words matching the following regular expression:
+       
+           (\A|\s)#(\w+)
+        
+    That is, any words that are #hashtag.
+    """
+    _DOC_ERRORS = ["zA"]
+    _pattern = re.compile(r"(\A|\s)#(\w+)")
+    def _skip(self,word):
+        if self._pattern.match(word):
+            return True
+        return False
+
 
 class HTMLChunker(Chunker):
     """Chunker for breaking up HTML documents into chunks of checkable text.
@@ -490,8 +554,8 @@ class HTMLChunker(Chunker):
             while offset < len(text) and text[offset] != "<":
                 offset += 1
             ePos = offset
-            self.offset = offset
-            # Return if chunk isnt empty
+            self._offset = offset
+            # Return if chunk isn't empty
             if(sPos < offset):
                 return (text[sPos:offset],sPos)
         raise StopIteration()
